@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -10,8 +10,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asmgen.ml 8930 2008-07-24 05:35:22Z frisch $ *)
-
 (* From lambda to assembly code *)
 
 open Format
@@ -37,6 +35,9 @@ let pass_dump_linear_if ppf flag message phrase =
   if !flag then fprintf ppf "*** %s@.%a@." message Printlinear.fundecl phrase;
   phrase
 
+let clambda_dump_if ppf ulambda =
+  if !dump_clambda then Printclambda.clambda ppf ulambda; ulambda
+
 let rec regalloc ppf round fd =
   if round > 50 then
     fatal_error(fd.Mach.fun_name ^
@@ -56,6 +57,7 @@ let rec regalloc ppf round fd =
 let (++) x f = f x
 
 let compile_fundecl (ppf : formatter) fd_cmm =
+  Proc.init ();
   Reg.reset();
   fd_cmm
   ++ Selection.fundecl
@@ -90,7 +92,7 @@ let compile_genfuns ppf f =
   List.iter
     (function
        | (Cfunction {fun_name = name}) as ph when f name ->
-	   compile_phrase ppf ph
+           compile_phrase ppf ph
        | _ -> ())
     (Cmmgen.generic_functions true [Compilenv.current_unit_infos ()])
 
@@ -104,6 +106,7 @@ let compile_implementation ?toplevel prefixname ppf (size, lam) =
     Emitaux.output_channel := oc;
     Emit.begin_assembly();
     Closure.intro size lam
+    ++ clambda_dump_if ppf
     ++ Cmmgen.compunit size
     ++ List.iter (compile_phrase ppf) ++ (fun () -> ());
     (match toplevel with None -> () | Some f -> compile_genfuns ppf f);
@@ -135,4 +138,5 @@ let compile_implementation ?toplevel prefixname ppf (size, lam) =
 
 let report_error ppf = function
   | Assembler_error file ->
-      fprintf ppf "Assembler error, input left in file %s" file
+      fprintf ppf "Assembler error, input left in file %a"
+        Location.print_filename file
