@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*         Jerome Vouillon, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -10,9 +10,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: translclass.ml,v 1.38.2.1 2006/03/08 02:05:42 garrigue Exp $ *)
-
-open Misc
 open Asttypes
 open Types
 open Typedtree
@@ -34,12 +31,14 @@ let lfunction params body =
   |  _ ->
       Lfunction (Curried, params, body)
 
-let lapply func args =
+let lapply func args loc =
   match func with
-    Lapply(func', args') ->
-      Lapply(func', args' @ args)
+    Lapply(func', args', _) ->
+      Lapply(func', args' @ args, loc)
   | _ ->
-      Lapply(func, args)
+      Lapply(func, args, loc)
+
+let mkappl (func, args) = Lapply (func, args, Location.none);;
 
 let lsequence l1 l2 =
   if l2 = lambda_unit then l1 else Lsequence(l1, l2)
@@ -48,7 +47,7 @@ let lfield v i = Lprim(Pfield i, [Lvar v])
 
 let transl_label l = share (Const_immstring l)
 
-let rec transl_meth_list lst =
+let transl_meth_list lst =
   if lst = [] then Lconst (Const_pointer 0) else
   share (Const_block
             (0, List.map (fun lab -> Const_immstring lab) lst))
@@ -68,13 +67,13 @@ let copy_inst_var obj id expr templ offset =
                                                     Lvar offset])])]))
 
 let transl_val tbl create name =
-  Lapply (oo_prim (if create then "new_variable" else "get_variable"),
+  mkappl (oo_prim (if create then "new_variable" else "get_variable"),
           [Lvar tbl; transl_label name])
 
-let transl_vals tbl create vals rem =
+let transl_vals tbl create strict vals rem =
   List.fold_right
     (fun (name, id) rem ->
-      Llet(StrictOpt, id, transl_val tbl create name, rem))
+      Llet(strict, id, transl_val tbl create name, rem))
     vals rem
 
 let meths_super tbl meths inh_meths =
@@ -82,13 +81,13 @@ let meths_super tbl meths inh_meths =
     (fun (nm, id) rem ->
        try
          (nm, id,
-          Lapply(oo_prim "get_method", [Lvar tbl; Lvar (Meths.find nm meths)]))
+          mkappl(oo_prim "get_method", [Lvar tbl; Lvar (Meths.find nm meths)]))
          :: rem
        with Not_found -> rem)
     inh_meths []
 
 let bind_super tbl (vals, meths) cl_init =
-  transl_vals tbl false vals
+  transl_vals tbl false StrictOpt vals
     (List.fold_right (fun (nm, id, def) rem -> Llet(StrictOpt, id, def, rem))
        meths cl_init)
 
@@ -97,22 +96,22 @@ let create_object cl obj init =
   let (inh_init, obj_init, has_init) = init obj' in
   if obj_init = lambda_unit then
     (inh_init,
-     Lapply (oo_prim (if has_init then "create_object_and_run_initializers"
+     mkappl (oo_prim (if has_init then "create_object_and_run_initializers"
                       else"create_object_opt"),
              [obj; Lvar cl]))
   else begin
    (inh_init,
     Llet(Strict, obj',
-            Lapply (oo_prim "create_object_opt", [obj; Lvar cl]),
+            mkappl (oo_prim "create_object_opt", [obj; Lvar cl]),
          Lsequence(obj_init,
                    if not has_init then Lvar obj' else
-                   Lapply (oo_prim "run_initializers_opt",
-			   [obj; Lvar obj'; Lvar cl]))))
+                   mkappl (oo_prim "run_initializers_opt",
+                           [obj; Lvar obj'; Lvar cl]))))
   end
 
 let rec build_object_init cl_table obj params inh_init obj_init cl =
   match cl.cl_desc with
-    Tclass_ident path ->
+    Tcl_ident ( path, _, _) ->
       let obj_init = Ident.create "obj_init" in
       let envs, inh_init = inh_init in
       let env =
@@ -120,36 +119,28 @@ let rec build_object_init cl_table obj params inh_init obj_init cl =
         | Some envs -> [Lprim(Pfield (List.length inh_init + 1), [Lvar envs])]
       in
       ((envs, (obj_init, path)::inh_init),
-       Lapply(Lvar obj_init, env @ [obj]))
-  | Tclass_structure str ->
+       mkappl(Lvar obj_init, env @ [obj]))
+  | Tcl_structure str ->
       create_object cl_table obj (fun obj ->
         let (inh_init, obj_init, has_init) =
           List.fold_right
             (fun field (inh_init, obj_init, has_init) ->
-               match field with
-                 Cf_inher (cl, _, _) ->
+               match field.cf_desc with
+                 Tcf_inher (_, cl, _, _, _) ->
                    let (inh_init, obj_init') =
                      build_object_init cl_table (Lvar obj) [] inh_init
                        (fun _ -> lambda_unit) cl
                    in
                    (inh_init, lsequence obj_init' obj_init, true)
-               | Cf_val (_, id, exp) ->
+               | Tcf_val (_, _, _, id, Tcfk_concrete exp, _) ->
                    (inh_init, lsequence (set_inst_var obj id exp) obj_init,
                     has_init)
-               | Cf_meth _ ->
+               | Tcf_meth _ | Tcf_val _ | Tcf_constr _ ->
                    (inh_init, obj_init, has_init)
-               | Cf_init _ ->
+               | Tcf_init _ ->
                    (inh_init, obj_init, true)
-               | Cf_let (rec_flag, defs, vals) ->
-                   (inh_init,
-                    Translcore.transl_let rec_flag defs
-                      (List.fold_right
-                         (fun (id, expr) rem ->
-                            lsequence (Lifused(id, set_inst_var obj id expr))
-                                      rem)
-                         vals obj_init),
-                    has_init))
-            str.cl_field
+            )
+            str.cstr_fields
             (inh_init, obj_init obj, false)
         in
         (inh_init,
@@ -158,7 +149,8 @@ let rec build_object_init cl_table obj params inh_init obj_init cl =
               lsequence (Lifused (id, set_inst_var obj id expr)) rem)
            params obj_init,
          has_init))
-  | Tclass_fun (pat, vals, cl, partial) ->
+  | Tcl_fun (_, pat, vals, cl, partial) ->
+      let vals = List.map (fun (id, _, e) -> id,e) vals in
       let (inh_init, obj_init) =
         build_object_init cl_table obj (vals @ params) inh_init obj_init cl
       in
@@ -173,22 +165,24 @@ let rec build_object_init cl_table obj params inh_init obj_init cl =
          Lfunction (Curried, params, rem) -> build params rem
        | rem                              -> build [] rem
        end)
-  | Tclass_apply (cl, oexprs) ->
+  | Tcl_apply (cl, oexprs) ->
       let (inh_init, obj_init) =
         build_object_init cl_table obj params inh_init obj_init cl
       in
-      (inh_init, transl_apply obj_init oexprs)
-  | Tclass_let (rec_flag, defs, vals, cl) ->
+      (inh_init, transl_apply obj_init oexprs Location.none)
+  | Tcl_let (rec_flag, defs, vals, cl) ->
+      let vals = List.map (fun (id, _, e) -> id,e) vals in
       let (inh_init, obj_init) =
         build_object_init cl_table obj (vals @ params) inh_init obj_init cl
       in
       (inh_init, Translcore.transl_let rec_flag defs obj_init)
-  | Tclass_constraint (cl, vals, pub_meths, concr_meths) ->
+  | Tcl_constraint (cl, _, vals, pub_meths, concr_meths) ->
       build_object_init cl_table obj params inh_init obj_init cl
 
 let rec build_object_init_0 cl_table params cl copy_env subst_env top ids =
   match cl.cl_desc with
-    Tclass_let (rec_flag, defs, vals, cl) ->
+    Tcl_let (rec_flag, defs, vals, cl) ->
+      let vals = List.map (fun (id, _, e) -> id,e) vals in
       build_object_init_0 cl_table (vals@params) cl copy_env subst_env top ids
   | _ ->
       let self = Ident.create "self" in
@@ -198,57 +192,47 @@ let rec build_object_init_0 cl_table params cl copy_env subst_env top ids =
       let ((_,inh_init), obj_init) =
         build_object_init cl_table obj params (envs,[]) (copy_env env) cl in
       let obj_init =
-	if ids = [] then obj_init else lfunction [self] obj_init in
+        if ids = [] then obj_init else lfunction [self] obj_init in
       (inh_init, lfunction [env] (subst_env env inh_init obj_init))
 
 
 let bind_method tbl lab id cl_init =
-  Llet(StrictOpt, id, Lapply (oo_prim "get_method_label",
-                              [Lvar tbl; transl_label lab]),
+  Llet(Strict, id, mkappl (oo_prim "get_method_label",
+                           [Lvar tbl; transl_label lab]),
        cl_init)
 
 let bind_methods tbl meths vals cl_init =
   let methl = Meths.fold (fun lab id tl -> (lab,id) :: tl) meths [] in
   let len = List.length methl and nvals = List.length vals in
   if len < 2 && nvals = 0 then Meths.fold (bind_method tbl) meths cl_init else
-  if len = 0 && nvals < 2 then transl_vals tbl true vals cl_init else
+  if len = 0 && nvals < 2 then transl_vals tbl true Strict vals cl_init else
   let ids = Ident.create "ids" in
-  let i = ref len in
-  let getter, names, cl_init =
-    match vals with [] -> "get_method_labels", [], cl_init
-    | (_,id0)::vals' ->
-        incr i;
-        let i = ref (List.length vals) in
-        "new_methods_variables",
-        [transl_meth_list (List.map fst vals)],
-        Llet(Strict, id0, lfield ids 0,
-	     List.fold_right
-	       (fun (name,id) rem ->
-	         decr i;
-                 Llet(Alias, id, Lprim(Poffsetint !i, [Lvar id0]), rem))
-	       vals' cl_init)
+  let i = ref (len + nvals) in
+  let getter, names =
+    if nvals = 0 then "get_method_labels", [] else
+    "new_methods_variables", [transl_meth_list (List.map fst vals)]
   in
-  Llet(StrictOpt, ids,
-       Lapply (oo_prim getter,
+  Llet(Strict, ids,
+       mkappl (oo_prim getter,
                [Lvar tbl; transl_meth_list (List.map fst methl)] @ names),
        List.fold_right
          (fun (lab,id) lam -> decr i; Llet(StrictOpt, id, lfield ids !i, lam))
-         methl cl_init)
+         (methl @ vals) cl_init)
 
 let output_methods tbl methods lam =
   match methods with
     [] -> lam
   | [lab; code] ->
-      lsequence (Lapply(oo_prim "set_method", [Lvar tbl; lab; code])) lam
+      lsequence (mkappl(oo_prim "set_method", [Lvar tbl; lab; code])) lam
   | _ ->
-      lsequence (Lapply(oo_prim "set_methods",
+      lsequence (mkappl(oo_prim "set_methods",
                         [Lvar tbl; Lprim(Pmakeblock(0,Immutable), methods)]))
         lam
 
 let rec ignore_cstrs cl =
   match cl.cl_desc with
-    Tclass_constraint (cl, _, _, _) -> ignore_cstrs cl
-  | Tclass_apply (cl, _) -> ignore_cstrs cl
+    Tcl_constraint (cl, _, _, _, _) -> ignore_cstrs cl
+  | Tcl_apply (cl, _) -> ignore_cstrs cl
   | _ -> cl
 
 let rec index a = function
@@ -256,36 +240,43 @@ let rec index a = function
   | b :: l ->
       if b = a then 0 else 1 + index a l
 
+let bind_id_as_val (id, _, _) = ("", id)
+
 let rec build_class_init cla cstr super inh_init cl_init msubst top cl =
   match cl.cl_desc with
-    Tclass_ident path ->
+    Tcl_ident ( path, _, _) ->
       begin match inh_init with
         (obj_init, path')::inh_init ->
-	  let lpath = transl_path path in
+          let lpath = transl_path path in
           (inh_init,
-           Llet (Strict, obj_init, 
-                 Lapply(Lprim(Pfield 1, [lpath]), Lvar cla ::
-			if top then [Lprim(Pfield 3, [lpath])] else []),
+           Llet (Strict, obj_init,
+                 mkappl(Lprim(Pfield 1, [lpath]), Lvar cla ::
+                        if top then [Lprim(Pfield 3, [lpath])] else []),
                  bind_super cla super cl_init))
       | _ ->
           assert false
       end
-  | Tclass_structure str ->
+  | Tcl_structure str ->
       let cl_init = bind_super cla super cl_init in
       let (inh_init, cl_init, methods, values) =
         List.fold_right
           (fun field (inh_init, cl_init, methods, values) ->
-            match field with
-              Cf_inher (cl, vals, meths) ->
+            match field.cf_desc with
+              Tcf_inher (_, cl, _, vals, meths) ->
                 let cl_init = output_methods cla methods cl_init in
                 let inh_init, cl_init =
                   build_class_init cla false
-                    (vals, meths_super cla str.cl_meths meths)
+                    (vals, meths_super cla str.cstr_meths meths)
                     inh_init cl_init msubst top cl in
                 (inh_init, cl_init, [], values)
-            | Cf_val (name, id, exp) ->
-                (inh_init, cl_init, methods, (name, id)::values)
-            | Cf_meth (name, exp) ->
+            | Tcf_val (name, _, _, id, exp, over) ->
+                let values = if over then values else (name, id) :: values in
+                (inh_init, cl_init, methods, values)
+            | Tcf_meth (_, _, _, Tcfk_virtual _, _)
+            | Tcf_constr _
+              ->
+                (inh_init, cl_init, methods, values)
+            | Tcf_meth (name, _, _, Tcfk_concrete exp, over) ->
                 let met_code = msubst true (transl_exp exp) in
                 let met_code =
                   if !Clflags.native_code && List.length met_code = 1 then
@@ -295,114 +286,115 @@ let rec build_class_init cla cstr super inh_init cl_init msubst top cl =
                   else met_code
                 in
                 (inh_init, cl_init,
-                 Lvar (Meths.find name str.cl_meths) :: met_code @ methods,
+                 Lvar (Meths.find name str.cstr_meths) :: met_code @ methods,
                  values)
-            | Cf_let (rec_flag, defs, vals) ->
-                let vals =
-                  List.map (function (id, _) -> (Ident.name id, id)) vals
-                in
-                (inh_init, cl_init, methods, vals @ values)
-            | Cf_init exp ->
+            | Tcf_init exp ->
                 (inh_init,
-                 Lsequence(Lapply (oo_prim "add_initializer",
+                 Lsequence(mkappl (oo_prim "add_initializer",
                                    Lvar cla :: msubst false (transl_exp exp)),
                            cl_init),
                  methods, values))
-          str.cl_field
+          str.cstr_fields
           (inh_init, cl_init, [], [])
       in
       let cl_init = output_methods cla methods cl_init in
-      (inh_init, bind_methods cla str.cl_meths values cl_init)
-  | Tclass_fun (pat, vals, cl, _) ->
+      (inh_init, bind_methods cla str.cstr_meths values cl_init)
+  | Tcl_fun (_, pat, vals, cl, _) ->
       let (inh_init, cl_init) =
         build_class_init cla cstr super inh_init cl_init msubst top cl
       in
-      let vals = List.map (function (id, _) -> (Ident.name id, id)) vals in
-      (inh_init, transl_vals cla true vals cl_init)
-  | Tclass_apply (cl, exprs) ->
+      let vals = List.map bind_id_as_val vals in
+      (inh_init, transl_vals cla true StrictOpt vals cl_init)
+  | Tcl_apply (cl, exprs) ->
       build_class_init cla cstr super inh_init cl_init msubst top cl
-  | Tclass_let (rec_flag, defs, vals, cl) ->
+  | Tcl_let (rec_flag, defs, vals, cl) ->
       let (inh_init, cl_init) =
         build_class_init cla cstr super inh_init cl_init msubst top cl
       in
-      let vals = List.map (function (id, _) -> (Ident.name id, id)) vals in
-      (inh_init, transl_vals cla true vals cl_init)
-  | Tclass_constraint (cl, vals, meths, concr_meths) ->
+      let vals = List.map bind_id_as_val vals in
+      (inh_init, transl_vals cla true StrictOpt vals cl_init)
+  | Tcl_constraint (cl, _, vals, meths, concr_meths) ->
       let virt_meths =
         List.filter (fun lab -> not (Concr.mem lab concr_meths)) meths in
       let concr_meths = Concr.elements concr_meths in
       let narrow_args =
-	[Lvar cla;
+        [Lvar cla;
          transl_meth_list vals;
          transl_meth_list virt_meths;
          transl_meth_list concr_meths] in
       let cl = ignore_cstrs cl in
       begin match cl.cl_desc, inh_init with
-	Tclass_ident path, (obj_init, path')::inh_init ->
-	  assert (Path.same path path');
-	  let lpath = transl_path path in
+        Tcl_ident (path, _, _), (obj_init, path')::inh_init ->
+          assert (Path.same path path');
+          let lpath = transl_path path in
           let inh = Ident.create "inh"
-          and inh_vals = Ident.create "vals"
-          and inh_meths = Ident.create "meths"
+          and ofs = List.length vals + 1
           and valids, methids = super in
           let cl_init =
             List.fold_left
               (fun init (nm, id, _) ->
-                Llet(StrictOpt, id, lfield inh_meths (index nm concr_meths),
+                Llet(StrictOpt, id, lfield inh (index nm concr_meths + ofs),
                      init))
               cl_init methids in
           let cl_init =
             List.fold_left
               (fun init (nm, id) ->
-                Llet(StrictOpt, id, lfield inh_vals (index nm vals), init))
+                Llet(StrictOpt, id, lfield inh (index nm vals + 1), init))
               cl_init valids in
           (inh_init,
-           Llet (Strict, inh, 
-		 Lapply(oo_prim "inherits", narrow_args @
-			[lpath; Lconst(Const_pointer(if top then 1 else 0))]),
-                 Llet(StrictOpt, obj_init, lfield inh 0,
-                 Llet(Alias, inh_vals, lfield inh 1,
-                 Llet(Alias, inh_meths, lfield inh 2, cl_init)))))
+           Llet (Strict, inh,
+                 mkappl(oo_prim "inherits", narrow_args @
+                        [lpath; Lconst(Const_pointer(if top then 1 else 0))]),
+                 Llet(StrictOpt, obj_init, lfield inh 0, cl_init)))
       | _ ->
-	  let core cl_init =
+          let core cl_init =
             build_class_init cla true super inh_init cl_init msubst top cl
-	  in
-	  if cstr then core cl_init else
+          in
+          if cstr then core cl_init else
           let (inh_init, cl_init) =
-            core (Lsequence (Lapply (oo_prim "widen", [Lvar cla]), cl_init))
+            core (Lsequence (mkappl (oo_prim "widen", [Lvar cla]), cl_init))
           in
           (inh_init,
-           Lsequence(Lapply (oo_prim "narrow", narrow_args), cl_init))
+           Lsequence(mkappl (oo_prim "narrow", narrow_args),
+                     cl_init))
       end
 
-let rec build_class_lets cl =
+let rec build_class_lets cl ids =
   match cl.cl_desc with
-    Tclass_let (rec_flag, defs, vals, cl) ->
-      let env, wrap = build_class_lets cl in
-      (env, fun x -> Translcore.transl_let rec_flag defs (wrap x))
+    Tcl_let (rec_flag, defs, vals, cl') ->
+      let env, wrap = build_class_lets cl' [] in
+      (env, fun x ->
+        let lam = Translcore.transl_let rec_flag defs (wrap x) in
+        (* Check recursion in toplevel let-definitions *)
+        if ids = [] || Translcore.check_recursive_lambda ids lam then lam
+        else raise(Error(cl.cl_loc, Illegal_class_expr)))
   | _ ->
       (cl.cl_env, fun x -> x)
 
 let rec get_class_meths cl =
   match cl.cl_desc with
-    Tclass_structure cl ->
-      Meths.fold (fun _ -> IdentSet.add) cl.cl_meths IdentSet.empty
-  | Tclass_ident _ -> IdentSet.empty
-  | Tclass_fun (_, _, cl, _)
-  | Tclass_let (_, _, _, cl)
-  | Tclass_apply (cl, _)
-  | Tclass_constraint (cl, _, _, _) -> get_class_meths cl
+    Tcl_structure cl ->
+      Meths.fold (fun _ -> IdentSet.add) cl.cstr_meths IdentSet.empty
+  | Tcl_ident _ -> IdentSet.empty
+  | Tcl_fun (_, _, _, cl, _)
+  | Tcl_let (_, _, _, cl)
+  | Tcl_apply (cl, _)
+  | Tcl_constraint (cl, _, _, _, _) -> get_class_meths cl
 
 (*
    XXX Il devrait etre peu couteux d'ecrire des classes :
      class c x y = d e f
 *)
-let rec transl_class_rebind obj_init cl =
+let rec transl_class_rebind obj_init cl vf =
   match cl.cl_desc with
-    Tclass_ident path ->
+    Tcl_ident (path, _, _) ->
+      if vf = Concrete then begin
+        try if (Env.find_class path cl.cl_env).cty_new = None then raise Exit
+        with Not_found -> raise Exit
+      end;
       (path, obj_init)
-  | Tclass_fun (pat, _, cl, partial) ->
-      let path, obj_init = transl_class_rebind obj_init cl in
+  | Tcl_fun (_, pat, _, cl, partial) ->
+      let path, obj_init = transl_class_rebind obj_init cl vf in
       let build params rem =
         let param = name_pattern "param" [pat, ()] in
         Lfunction (Curried, param::params,
@@ -413,38 +405,38 @@ let rec transl_class_rebind obj_init cl =
        match obj_init with
          Lfunction (Curried, params, rem) -> build params rem
        | rem                              -> build [] rem)
-  | Tclass_apply (cl, oexprs) ->
-      let path, obj_init = transl_class_rebind obj_init cl in
-      (path, transl_apply obj_init oexprs)
-  | Tclass_let (rec_flag, defs, vals, cl) ->
-      let path, obj_init = transl_class_rebind obj_init cl in
+  | Tcl_apply (cl, oexprs) ->
+      let path, obj_init = transl_class_rebind obj_init cl vf in
+      (path, transl_apply obj_init oexprs Location.none)
+  | Tcl_let (rec_flag, defs, vals, cl) ->
+      let path, obj_init = transl_class_rebind obj_init cl vf in
       (path, Translcore.transl_let rec_flag defs obj_init)
-  | Tclass_structure _ -> raise Exit
-  | Tclass_constraint (cl', _, _, _) ->
-      let path, obj_init = transl_class_rebind obj_init cl' in
+  | Tcl_structure _ -> raise Exit
+  | Tcl_constraint (cl', _, _, _, _) ->
+      let path, obj_init = transl_class_rebind obj_init cl' vf in
       let rec check_constraint = function
-          Tcty_constr(path', _, _) when Path.same path path' -> ()
-        | Tcty_fun (_, _, cty) -> check_constraint cty
+          Cty_constr(path', _, _) when Path.same path path' -> ()
+        | Cty_fun (_, _, cty) -> check_constraint cty
         | _ -> raise Exit
       in
       check_constraint cl.cl_type;
       (path, obj_init)
 
-let rec transl_class_rebind_0 self obj_init cl =
+let rec transl_class_rebind_0 self obj_init cl vf =
   match cl.cl_desc with
-    Tclass_let (rec_flag, defs, vals, cl) ->
-      let path, obj_init = transl_class_rebind_0 self obj_init cl in
+    Tcl_let (rec_flag, defs, vals, cl) ->
+      let path, obj_init = transl_class_rebind_0 self obj_init cl vf in
       (path, Translcore.transl_let rec_flag defs obj_init)
   | _ ->
-      let path, obj_init = transl_class_rebind obj_init cl in
+      let path, obj_init = transl_class_rebind obj_init cl vf in
       (path, lfunction [self] obj_init)
 
-let transl_class_rebind ids cl =
+let transl_class_rebind ids cl vf =
   try
     let obj_init = Ident.create "obj_init"
     and self = Ident.create "self" in
-    let obj_init0 = lapply (Lvar obj_init) [Lvar self] in
-    let path, obj_init' = transl_class_rebind_0 self obj_init0 cl in
+    let obj_init0 = lapply (Lvar obj_init) [Lvar self] Location.none in
+    let path, obj_init' = transl_class_rebind_0 self obj_init0 cl vf in
     if not (Translcore.check_recursive_lambda ids obj_init') then
       raise(Error(cl.cl_loc, Illegal_class_expr));
     let id = (obj_init' = lfunction [self] obj_init0) in
@@ -460,13 +452,13 @@ let transl_class_rebind ids cl =
     Llet(
     Alias, cla, transl_path path,
     Lprim(Pmakeblock(0, Immutable),
-          [Lapply(Lvar new_init, [lfield cla 0]);
+          [mkappl(Lvar new_init, [lfield cla 0]);
            lfunction [table]
              (Llet(Strict, env_init,
-                   Lapply(lfield cla 1, [Lvar table]),
+                   mkappl(lfield cla 1, [Lvar table]),
                    lfunction [envs]
-                     (Lapply(Lvar new_init,
-                             [Lapply(Lvar env_init, [Lvar envs])]))));
+                     (mkappl(Lvar new_init,
+                             [mkappl(Lvar env_init, [Lvar envs])]))));
            lfield cla 2;
            lfield cla 3])))
   with Exit ->
@@ -498,30 +490,30 @@ let rec builtin_meths self env env2 body =
         "var", [Lvar n]
     | Lprim(Pfield n, [Lvar e]) when Ident.same e env ->
         "env", [Lvar env2; Lconst(Const_pointer n)]
-    | Lsend(Self, met, Lvar s, []) when List.mem s self ->
+    | Lsend(Self, met, Lvar s, [], _) when List.mem s self ->
         "meth", [met]
     | _ -> raise Not_found
   in
   match body with
   | Llet(_, s', Lvar s, body) when List.mem s self ->
       builtin_meths (s'::self) env env2 body
-  | Lapply(f, [arg]) when const_path f ->
+  | Lapply(f, [arg], _) when const_path f ->
       let s, args = conv arg in ("app_"^s, f :: args)
-  | Lapply(f, [arg; p]) when const_path f && const_path p ->
+  | Lapply(f, [arg; p], _) when const_path f && const_path p ->
       let s, args = conv arg in
       ("app_"^s^"_const", f :: args @ [p])
-  | Lapply(f, [p; arg]) when const_path f && const_path p ->
+  | Lapply(f, [p; arg], _) when const_path f && const_path p ->
       let s, args = conv arg in
       ("app_const_"^s, f :: p :: args)
-  | Lsend(Self, Lvar n, Lvar s, [arg]) when List.mem s self ->
+  | Lsend(Self, Lvar n, Lvar s, [arg], _) when List.mem s self ->
       let s, args = conv arg in
       ("meth_app_"^s, Lvar n :: args)
-  | Lsend(Self, met, Lvar s, []) when List.mem s self ->
+  | Lsend(Self, met, Lvar s, [], _) when List.mem s self ->
       ("get_meth", [met])
-  | Lsend(Public, met, arg, []) ->
+  | Lsend(Public, met, arg, [], _) ->
       let s, args = conv arg in
       ("send_"^s, met :: args)
-  | Lsend(Cached, met, arg, [_;_]) ->
+  | Lsend(Cached, met, arg, [_;_], _) ->
       let s, args = conv arg in
       ("send_"^s, met :: args)
   | Lfunction (Curried, [x], body) ->
@@ -539,9 +531,9 @@ let rec builtin_meths self env env2 body =
 
 module M = struct
   open CamlinternalOO
-  let builtin_meths arr self env env2 body =
+  let builtin_meths self env env2 body =
     let builtin, args = builtin_meths self env env2 body in
-    if not arr then [Lapply(oo_prim builtin, args)] else
+    (* if not arr then [mkappl(oo_prim builtin, args)] else *)
     let tag = match builtin with
       "get_const" -> GetConst
     | "get_var"   -> GetVar
@@ -591,32 +583,39 @@ open M
    Si ids=0 (objet immediat), alors on ne conserve que env_init.
 *)
 
+let prerr_ids msg ids =
+  let names = List.map Ident.unique_toplevel_name ids in
+  prerr_endline (String.concat " " (msg :: names))
 
-let transl_class ids cl_id arity pub_meths cl =
+let transl_class ids cl_id pub_meths cl vflag =
   (* First check if it is not only a rebind *)
-  let rebind = transl_class_rebind ids cl in
+  let rebind = transl_class_rebind ids cl vflag in
   if rebind <> lambda_unit then rebind else
 
   (* Prepare for heavy environment handling *)
   let tables = Ident.create (Ident.name cl_id ^ "_tables") in
   let (top_env, req) = oo_add_class tables in
   let top = not req in
-  let cl_env, llets = build_class_lets cl in
+  let cl_env, llets = build_class_lets cl ids in
   let new_ids = if top then [] else Env.diff top_env cl_env in
   let env2 = Ident.create "env" in
   let meth_ids = get_class_meths cl in
   let subst env lam i0 new_ids' =
     let fv = free_variables lam in
+    (* prerr_ids "cl_id =" [cl_id]; prerr_ids "fv =" (IdentSet.elements fv); *)
     let fv = List.fold_right IdentSet.remove !new_ids' fv in
-    (* IdentSet.iter
-      (fun id ->
-        if not (List.mem id new_ids) then prerr_endline (Ident.name id))
-      fv; *)
-    let fv = IdentSet.filter (fun id -> List.mem id new_ids) fv in
-    (* need to handle methods specially (PR#3576) *)
-    let fm = IdentSet.diff (free_methods lam) meth_ids in
-    let fv = IdentSet.union fv fm in
+    (* We need to handle method ids specially, as they do not appear
+       in the typing environment (PR#3576, PR#4560) *)
+    (* very hacky: we add and remove free method ids on the fly,
+       depending on the visit order... *)
+    method_ids :=
+      IdentSet.diff (IdentSet.union (free_methods lam) !method_ids) meth_ids;
+    (* prerr_ids "meth_ids =" (IdentSet.elements meth_ids);
+       prerr_ids "method_ids =" (IdentSet.elements !method_ids); *)
+    let new_ids = List.fold_right IdentSet.add new_ids !method_ids in
+    let fv = IdentSet.inter fv new_ids in
     new_ids' := !new_ids' @ IdentSet.elements fv;
+    (* prerr_ids "new_ids' =" !new_ids'; *)
     let i = ref (i0-1) in
     List.fold_left
       (fun subst id ->
@@ -633,8 +632,8 @@ let transl_class ids cl_id arity pub_meths cl =
         begin try
           (* Doesn't seem to improve size for bytecode *)
           (* if not !Clflags.native_code then raise Not_found; *)
-          if !Clflags.debug then raise Not_found;
-          builtin_meths arr [self] env env2 (lfunction args body')
+          if not arr || !Clflags.debug then raise Not_found;
+          builtin_meths [self] env env2 (lfunction args body')
         with Not_found ->
           [lfunction (self :: args)
              (if not (IdentSet.mem env (free_variables body')) then body' else
@@ -664,8 +663,6 @@ let transl_class ids cl_id arity pub_meths cl =
   let cla = Ident.create "class" in
   let (inh_init, obj_init) =
     build_object_init_0 cla [] cl copy_env subst_env top ids in
-  if not (Translcore.check_recursive_lambda ids obj_init) then
-    raise(Error(cl.cl_loc, Illegal_class_expr));
   let inh_init' = List.rev inh_init in
   let (inh_init', cl_init) =
     build_class_init cla true ([],[]) inh_init' obj_init msubst top cl
@@ -688,34 +685,32 @@ let transl_class ids cl_id arity pub_meths cl =
     tags pub_meths;
   let ltable table lam =
     Llet(Strict, table,
-         Lapply (oo_prim "create_table", [transl_meth_list pub_meths]), lam)
+         mkappl (oo_prim "create_table", [transl_meth_list pub_meths]), lam)
   and ldirect obj_init =
     Llet(Strict, obj_init, cl_init,
-         Lsequence(Lapply (oo_prim "init_class", [Lvar cla]),
-                   Lapply(Lvar obj_init, [lambda_unit])))
+         Lsequence(mkappl (oo_prim "init_class", [Lvar cla]),
+                   mkappl (Lvar obj_init, [lambda_unit])))
   in
   (* Simplest case: an object defined at toplevel (ids=[]) *)
   if top && ids = [] then llets (ltable cla (ldirect obj_init)) else
 
-  let concrete =
-    ids = [] ||
-    Typeclass.virtual_methods (Ctype.signature_of_class_type cl.cl_type) = []
+  let concrete = (vflag = Concrete)
   and lclass lam =
     let cl_init = llets (Lfunction(Curried, [cla], cl_init)) in
     Llet(Strict, class_init, cl_init, lam (free_variables cl_init))
   and lbody fv =
     if List.for_all (fun id -> not (IdentSet.mem id fv)) ids then
-      Lapply (oo_prim "make_class",[transl_meth_list pub_meths;
-				    Lvar class_init])
+      mkappl (oo_prim "make_class",[transl_meth_list pub_meths;
+                                    Lvar class_init])
     else
       ltable table (
       Llet(
-      Strict, env_init, Lapply(Lvar class_init, [Lvar table]),
+      Strict, env_init, mkappl (Lvar class_init, [Lvar table]),
       Lsequence(
-      Lapply (oo_prim "init_class", [Lvar table]),
+      mkappl (oo_prim "init_class", [Lvar table]),
       Lprim(Pmakeblock(0, Immutable),
-	    [Lapply(Lvar env_init, [lambda_unit]);
-	     Lvar class_init; Lvar env_init; lambda_unit]))))
+            [mkappl (Lvar env_init, [lambda_unit]);
+             Lvar class_init; Lvar env_init; lambda_unit]))))
   and lbody_virt lenvs =
     Lprim(Pmakeblock(0, Immutable),
           [lambda_unit; Lfunction(Curried,[cla], cl_init); lambda_unit; lenvs])
@@ -750,7 +745,7 @@ let transl_class ids cl_id arity pub_meths cl =
          lam)
   and def_ids cla lam =
     Llet(StrictOpt, env2,
-         Lapply (oo_prim "new_variable", [Lvar cla; transl_label ""]),
+         mkappl (oo_prim "new_variable", [Lvar cla; transl_label ""]),
          lam)
   in
   let inh_paths =
@@ -764,7 +759,7 @@ let transl_class ids cl_id arity pub_meths cl =
   and lcache lam =
     if inh_keys = [] then Llet(Alias, cached, Lvar tables, lam) else
     Llet(Strict, cached,
-         Lapply(oo_prim "lookup_tables",
+         mkappl (oo_prim "lookup_tables",
                 [Lvar tables; Lprim(Pmakeblock(0, Immutable), inh_keys)]),
          lam)
   and lset cached i lam =
@@ -773,7 +768,7 @@ let transl_class ids cl_id arity pub_meths cl =
   let ldirect () =
     ltable cla
       (Llet(Strict, env_init, def_ids cla cl_init,
-            Lsequence(Lapply (oo_prim "init_class", [Lvar cla]),
+            Lsequence(mkappl (oo_prim "init_class", [Lvar cla]),
                       lset cached 0 (Lvar env_init))))
   and lclass_virt () =
     lset cached 0 (Lfunction(Curried, [cla], def_ids cla cl_init))
@@ -785,14 +780,14 @@ let transl_class ids cl_id arity pub_meths cl =
               if ids = [] then ldirect () else
               if not concrete then lclass_virt () else
               lclass (
-              Lapply (oo_prim "make_class_store",
+              mkappl (oo_prim "make_class_store",
                       [transl_meth_list pub_meths;
                        Lvar class_init; Lvar cached]))),
   make_envs (
-  if ids = [] then Lapply(lfield cached 0, [lenvs]) else
+  if ids = [] then mkappl (lfield cached 0, [lenvs]) else
   Lprim(Pmakeblock(0, Immutable),
         if concrete then
-          [Lapply(lfield cached 0, [lenvs]);
+          [mkappl (lfield cached 0, [lenvs]);
            lfield cached 1;
            lfield cached 0;
            lenvs]
@@ -800,12 +795,20 @@ let transl_class ids cl_id arity pub_meths cl =
        )))))
 
 (* Wrapper for class compilation *)
+(*
+    let cl_id = ci.ci_id_class in
+(* TODO: cl_id is used somewhere else as typesharp ? *)
+  let _arity = List.length (fst ci.ci_params) in
+  let pub_meths = m in
+  let cl = ci.ci_expr in
+  let vflag = vf in
+*)
 
-let transl_class ids cl_id arity pub_meths cl =
-  oo_wrap cl.cl_env false (transl_class ids cl_id arity pub_meths) cl
+let transl_class ids id pub_meths cl vf =
+  oo_wrap cl.cl_env false (transl_class ids id pub_meths cl) vf
 
 let () =
-  transl_object := (fun id meths cl -> transl_class [] id 0 meths cl)
+  transl_object := (fun id meths cl -> transl_class [] id meths cl Concrete)
 
 (* Error report *)
 
@@ -813,7 +816,7 @@ open Format
 
 let report_error ppf = function
   | Illegal_class_expr ->
-      fprintf ppf "This kind of class expression is not allowed"
+      fprintf ppf "This kind of recursive class expression is not allowed"
   | Tags (lab1, lab2) ->
       fprintf ppf "Method labels `%s' and `%s' are incompatible.@ %s"
         lab1 lab2 "Change one of them."
