@@ -11,8 +11,6 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: dynlink.c 12677 2012-07-09 14:15:48Z doligez $ */
-
 /* Dynamic loading of C primitives. */
 
 #include <stddef.h>
@@ -165,7 +163,7 @@ void caml_build_primitive_table(char * lib_path,
   for (p = req_prims; *p != 0; p += strlen(p) + 1) {
     c_primitive prim = lookup_primitive(p);
     if (prim == NULL)
-      caml_fatal_error_arg("Fatal error: unknown C primitive `%s'\n", p);
+          caml_fatal_error_arg("Fatal error: unknown C primitive `%s'\n", p);
     caml_ext_table_add(&caml_prim_table, (void *) prim);
 #ifdef DEBUG
     caml_ext_table_add(&caml_prim_name_table, strdup(p));
@@ -190,7 +188,8 @@ void caml_build_primitive_table_builtin(void)
   for (i = 0; caml_builtin_cprim[i] != 0; i++) {
     caml_ext_table_add(&caml_prim_table, (void *) caml_builtin_cprim[i]);
 #ifdef DEBUG
-    caml_ext_table_add(&caml_prim_name_table, strdup(caml_names_of_builtin_cprim[i]));
+    caml_ext_table_add(&caml_prim_name_table,
+                       strdup(caml_names_of_builtin_cprim[i]));
 #endif
 }
 }
