@@ -1,6 +1,6 @@
 /***********************************************************************/
 /*                                                                     */
-/*                           Objective Caml                            */
+/*                                OCaml                                */
 /*                                                                     */
 /*             Damien Doligez, projet Para, INRIA Rocquencourt         */
 /*                                                                     */
@@ -11,8 +11,6 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: freelist.h,v 1.12 2004/01/02 19:23:21 doligez Exp $ */
-
 /* Free lists of heap blocks. */
 
 #ifndef CAML_FREELIST_H
@@ -28,8 +26,9 @@ char *caml_fl_allocate (mlsize_t);
 void caml_fl_init_merge (void);
 void caml_fl_reset (void);
 char *caml_fl_merge_block (char *);
-void caml_fl_add_block (char *);
-void caml_make_free_blocks (value *, mlsize_t, int);
+void caml_fl_add_blocks (char *);
+void caml_make_free_blocks (value *, mlsize_t, int, int);
+void caml_set_allocation_policy (uintnat);
 
 
 #endif /* CAML_FREELIST_H */
