@@ -1,6 +1,6 @@
 /***********************************************************************/
 /*                                                                     */
-/*                           Objective Caml                            */
+/*                                OCaml                                */
 /*                                                                     */
 /*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
 /*                                                                     */
@@ -11,8 +11,6 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: interp.c,v 1.96 2006/08/18 14:51:59 xleroy Exp $ */
-
 /* The bytecode interpreter */
 #include <stdio.h>
 #include "alloc.h"
@@ -66,9 +64,12 @@ sp is a local copy of the global variable caml_extern_sp. */
 
 #define Setup_for_gc \
   { sp -= 2; sp[0] = accu; sp[1] = env; caml_extern_sp = sp; }
-#define Restore_after_gc { accu = sp[0]; env = sp[1]; sp += 2; }
-#define Setup_for_c_call { saved_pc = pc; *--sp = env; caml_extern_sp = sp; }
-#define Restore_after_c_call { sp = caml_extern_sp; env = *sp++; }
+#define Restore_after_gc \
+  { accu = sp[0]; env = sp[1]; sp += 2; }
+#define Setup_for_c_call \
+  { saved_pc = pc; *--sp = env; caml_extern_sp = sp; }
+#define Restore_after_c_call \
+  { sp = caml_extern_sp; env = *sp++; saved_pc = NULL; }
 
 /* An event frame must look like accu + a C_CALL frame + a RETURN 1 frame */
 #define Setup_for_event \
@@ -110,7 +111,8 @@ sp is a local copy of the global variable caml_extern_sp. */
    For GCC, I have hand-assigned hardware registers for several architectures.
 */
 
-#if defined(__GNUC__) && !defined(__INTEL_COMPILER) && !defined(DEBUG)
+#if defined(__GNUC__) && !defined(DEBUG) && !defined(__INTEL_COMPILER) \
+    && !defined(__llvm__)
 #ifdef __mips__
 #define PC_REG asm("$16")
 #define SP_REG asm("$17")
@@ -154,8 +156,9 @@ sp is a local copy of the global variable caml_extern_sp. */
 #define SP_REG asm("a4")
 #define ACCU_REG asm("d7")
 #endif
-#ifdef __arm__
-#define PC_REG asm("r9")
+/* PR#4953: these specific registers not available in Thumb mode */
+#if defined (__arm__) && !defined(__thumb__)
+#define PC_REG asm("r6")
 #define SP_REG asm("r8")
 #define ACCU_REG asm("r7")
 #endif
@@ -211,9 +214,8 @@ value caml_interprete(code_t prog, asize_t prog_size)
   /* volatile ensures that initial_local_roots and saved_pc
      will keep correct value across longjmp */
   struct caml__roots_block * volatile initial_local_roots;
-  volatile code_t saved_pc;
+  volatile code_t saved_pc = NULL;
   struct longjmp_buffer raise_buf;
-  value * modify_dest, modify_newval;
 #ifndef THREADED_CODE
   opcode_t curr_instr;
 #endif
@@ -245,7 +247,9 @@ value caml_interprete(code_t prog, asize_t prog_size)
     caml_local_roots = initial_local_roots;
     sp = caml_extern_sp;
     accu = caml_exn_bucket;
-    pc = saved_pc + 2; /* +2 adjustement for the sole purpose of backtraces */
+    pc = saved_pc; saved_pc = NULL;
+    if (pc != NULL) pc += 2;
+        /* +2 adjustement for the sole purpose of backtraces */
     goto raise_exception;
   }
   caml_external_raise = &raise_buf;
@@ -701,29 +705,26 @@ value caml_interprete(code_t prog, asize_t prog_size)
     }
 
     Instruct(SETFIELD0):
-      modify_dest = &Field(accu, 0);
-      modify_newval = *sp++;
-    modify:
-      Modify(modify_dest, modify_newval);
+      caml_modify(&Field(accu, 0), *sp++);
       accu = Val_unit;
       Next;
     Instruct(SETFIELD1):
-      modify_dest = &Field(accu, 1);
-      modify_newval = *sp++;
-      goto modify;
+      caml_modify(&Field(accu, 1), *sp++);
+      accu = Val_unit;
+      Next;
     Instruct(SETFIELD2):
-      modify_dest = &Field(accu, 2);
-      modify_newval = *sp++;
-      goto modify;
+      caml_modify(&Field(accu, 2), *sp++);
+      accu = Val_unit;
+      Next;
     Instruct(SETFIELD3):
-      modify_dest = &Field(accu, 3);
-      modify_newval = *sp++;
-      goto modify;
+      caml_modify(&Field(accu, 3), *sp++);
+      accu = Val_unit;
+      Next;
     Instruct(SETFIELD):
-      modify_dest = &Field(accu, *pc);
+      caml_modify(&Field(accu, *pc), *sp++);
+      accu = Val_unit;
       pc++;
-      modify_newval = *sp++;
-      goto modify;
+      Next;
     Instruct(SETFLOATFIELD):
       Store_double_field(accu, *pc, Double_val(*sp));
       accu = Val_unit;
@@ -744,10 +745,10 @@ value caml_interprete(code_t prog, asize_t prog_size)
       sp += 1;
       Next;
     Instruct(SETVECTITEM):
-      modify_dest = &Field(accu, Long_val(sp[0]));
-      modify_newval = sp[1];
+      caml_modify(&Field(accu, Long_val(sp[0])), sp[1]);
+      accu = Val_unit;
       sp += 2;
-      goto modify;
+      Next;
 
 /* String operations */
 
@@ -1018,14 +1019,14 @@ value caml_interprete(code_t prog, asize_t prog_size)
     Instruct(ISINT):
       accu = Val_long(accu & 1);
       Next;
-    
+
 /* Object-oriented operations */
 
 #define Lookup(obj, lab) Field (Field (obj, 0), Int_val(lab))
 
       /* please don't forget to keep below code in sync with the
-	 functions caml_cache_public_method and
-	 caml_cache_public_method2 in obj.c */
+         functions caml_cache_public_method and
+         caml_cache_public_method2 in obj.c */
 
     Instruct(GETMETHOD):
       accu = Lookup(sp[0], accu);
@@ -1117,7 +1118,7 @@ value caml_interprete(code_t prog, asize_t prog_size)
 #else
       caml_fatal_error_arg("Fatal error: bad opcode (%"
                            ARCH_INTNAT_PRINTF_FORMAT "x)\n",
-                           (char *)(*(pc-1)));
+                           (char *) (intnat) *(pc-1));
 #endif
     }
   }
@@ -1131,7 +1132,7 @@ void caml_prepare_bytecode(code_t prog, asize_t prog_size) {
   Assert(prog);
   Assert(prog_size>0);
   /* actually, the threading of the bytecode might be done here */
-} 
+}
 
 void caml_release_bytecode(code_t prog, asize_t prog_size) {
   /* other implementations of the interpreter (such as an hypothetical
