@@ -11,8 +11,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: eval.ml 12511 2012-05-30 13:29:48Z lefessan $ *)
-
 open Misc
 open Path
 open Instruct
@@ -89,7 +87,8 @@ let rec expression event env = function
       end
   | E_result ->
       begin match event with
-        Some {ev_kind = Event_after ty; ev_typsubst = subst} when !Frames.current_frame = 0 ->
+        Some {ev_kind = Event_after ty; ev_typsubst = subst}
+        when !Frames.current_frame = 0 ->
           (Debugcom.Remote_value.accu(), Subst.type_expr subst ty)
       | _ ->
           raise(Error(No_result))
@@ -183,10 +182,12 @@ let report_error ppf = function
         pos len Printtyp.type_expr ty
   | Array_index(len, pos) ->
       fprintf ppf
-        "@[Cannot extract element number %i from an array of length %i@]@." pos len
+        "@[Cannot extract element number %i from an array of length %i@]@."
+        pos len
   | List_index(len, pos) ->
       fprintf ppf
-        "@[Cannot extract element number %i from a list of length %i@]@." pos len
+        "@[Cannot extract element number %i from a list of length %i@]@."
+        pos len
   | String_index(s, len, pos) ->
       fprintf ppf
         "@[Cannot extract character number %i@ \
