@@ -1,9 +1,9 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*          Jerome Vouillon, projet Cristal, INRIA Rocquencourt        *)
-(*          Objective Caml port by John Malecki and Xavier Leroy       *)
+(*          OCaml port by John Malecki and Xavier Leroy                *)
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
@@ -11,13 +11,11 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parameters.ml,v 1.3 2002/02/14 15:17:10 doligez Exp $ *)
-
 (* Miscellaneous parameters *)
 
 open Primitives
 open Config
-open Misc
+open Debugger_config
 
 let program_loaded = ref false
 let program_name = ref ""
@@ -31,5 +29,9 @@ let add_path dir =
   load_path := dir :: except dir !load_path;
   Envaux.reset_cache()
 
+let add_path_for mdl dir =
+  let old = try Hashtbl.find load_path_for mdl with Not_found -> [] in
+  Hashtbl.replace load_path_for mdl (dir :: old)
+
 (* Used by emacs ? *)
 let emacs = ref false
