@@ -1,9 +1,9 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*          Jerome Vouillon, projet Cristal, INRIA Rocquencourt        *)
-(*          Objective Caml port by John Malecki and Xavier Leroy       *)
+(*          OCaml port by John Malecki and Xavier Leroy                *)
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
@@ -11,8 +11,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: time_travel.ml,v 1.19 2005/08/25 15:35:16 doligez Exp $ *)
-
 (**************************** Time travel ******************************)
 
 open Int64ops
@@ -26,8 +24,10 @@ open Trap_barrier
 open Input_handling
 open Debugger_config
 open Program_loading
+open Question
 
 exception Current_checkpoint_lost
+exception Current_checkpoint_lost_start_at of int64 * int64
 
 let remove_1st key list =
   let rec remove =
@@ -92,7 +92,7 @@ let wait_for_connection checkpoint =
 (* Select a checkpoint as current. *)
 let set_current_checkpoint checkpoint =
   if !debug_time_travel then
-    prerr_endline ("Select : " ^ (string_of_int checkpoint.c_pid));
+    prerr_endline ("Select: " ^ (string_of_int checkpoint.c_pid));
   if not checkpoint.c_valid then
     wait_for_connection checkpoint;
   current_checkpoint := checkpoint;
@@ -101,7 +101,7 @@ let set_current_checkpoint checkpoint =
 (* Kill `checkpoint'. *)
 let kill_checkpoint checkpoint =
   if !debug_time_travel then
-    prerr_endline ("Kill : " ^ (string_of_int checkpoint.c_pid));
+    prerr_endline ("Kill: " ^ (string_of_int checkpoint.c_pid));
   if checkpoint.c_pid > 0 then          (* Ghosts don't have to be killed ! *)
     (if not checkpoint.c_valid then
        wait_for_connection checkpoint;
@@ -238,7 +238,7 @@ let duplicate_current_checkpoint () =
            Checkpoint_done pid ->
              (new_checkpoint.c_pid <- pid;
               if !debug_time_travel then
-                prerr_endline ("Waiting for connection : " ^ (string_of_int pid)))
+                prerr_endline ("Waiting for connection: " ^ string_of_int pid))
          | Checkpoint_failed ->
              prerr_endline
                "A fork failed. Reducing maximum number of checkpoints.";
@@ -324,7 +324,7 @@ let internal_step duration =
                   set_current_checkpoint
                     (find_checkpoint_before (current_time ()))));
         if !debug_time_travel then begin
-          print_string "Checkpoints : pid(time)"; print_newline ();
+          print_string "Checkpoints: pid(time)"; print_newline ();
           List.iter
             (function {c_time = time; c_pid = pid; c_valid = valid} ->
               Printf.printf "%d(%Ld)%s " pid time
@@ -370,7 +370,7 @@ let set_file_descriptor pid fd =
            true)
   in
     if !debug_time_travel then
-      prerr_endline ("New connection : " ^(string_of_int pid));
+      prerr_endline ("New connection: " ^(string_of_int pid));
     find (!current_checkpoint::!checkpoints)
 
 (* Kill all the checkpoints. *)
@@ -382,19 +382,24 @@ let kill_all_checkpoints () =
 (* --- Assume that the checkpoint is valid. *)
 let forget_process fd pid =
   let checkpoint =
-    find (function c -> c.c_pid = pid) (!current_checkpoint::!checkpoints)
+    List.find (function c -> c.c_pid = pid) (!current_checkpoint::!checkpoints)
   in
     Printf.eprintf "Lost connection with process %d" pid;
-    if checkpoint == !current_checkpoint then begin
-      Printf.eprintf " (active process)\n";
-      match !current_checkpoint.c_state with
-        C_stopped ->
-          Printf.eprintf "at time %Ld" !current_checkpoint.c_time
-      | C_running duration ->
-          Printf.eprintf "between time %Ld and time %Ld"
-                         !current_checkpoint.c_time
-                         (!current_checkpoint.c_time ++ duration)
-      end;
+    let kont =
+      if checkpoint == !current_checkpoint then begin
+        Printf.eprintf " (active process)\n";
+        match !current_checkpoint.c_state with
+          C_stopped ->
+            Printf.eprintf "at time %Ld" !current_checkpoint.c_time;
+            fun () -> raise Current_checkpoint_lost
+        | C_running duration ->
+            Printf.eprintf "between time %Ld and time %Ld"
+                          !current_checkpoint.c_time
+                          (!current_checkpoint.c_time ++ duration);
+            fun () -> raise (Current_checkpoint_lost_start_at
+                              (!current_checkpoint.c_time, duration))
+        end
+      else ignore in
     Printf.eprintf "\n"; flush stderr;
     Input_handling.remove_file fd;
     close_io checkpoint.c_fd;
@@ -403,8 +408,7 @@ let forget_process fd pid =
     checkpoint.c_pid <- -1;             (* Don't exist anymore *)
     if checkpoint.c_parent.c_pid > 0 then
       wait_child checkpoint.c_parent.c_fd;
-    if checkpoint == !current_checkpoint then
-      raise Current_checkpoint_lost
+    kont ()
 
 (* Try to recover when the current checkpoint is lost. *)
 let recover () =
