@@ -1,15 +1,15 @@
 (***********************************************************************)
-(*                           OCamldoc                                  *)
+(*                                                                     *)
+(*                             OCamldoc                                *)
 (*                                                                     *)
 (*      Olivier Andrieu, base sur du code de Maxence Guesdon           *)
 (*                                                                     *)
 (*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
 (*  under the terms of the Q Public License version 1.0.               *)
+(*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_texi.ml,v 1.19.4.3 2006/07/31 14:19:35 guesdon Exp $ *)
-
 (** Generation of Texinfo documentation. *)
 
 open Odoc_info
@@ -20,6 +20,12 @@ open Exception
 open Class
 open Module
 
+let esc_8bits = ref false
+
+let info_section = ref "OCaml"
+
+let info_entry = ref []
+
 (** {2 Some small helper functions} *)
 
 let puts_nl chan s =
@@ -140,25 +146,25 @@ struct
     (Str.regexp "}", "@}") ;
     (Str.regexp "\\.\\.\\.", "@dots{}") ;
   ] @
-    (if !Args.esc_8bits
+    (if !esc_8bits
     then [
-    (Str.regexp "", "@`a") ;
-    (Str.regexp "", "@^a") ;
-    (Str.regexp "", "@'e") ;
-    (Str.regexp "", "@`e") ;
-    (Str.regexp "", "@^e") ;
-    (Str.regexp "", "@\"e") ;
-    (Str.regexp "", "@,{c}") ;
-    (Str.regexp "", "@^o") ;
-    (Str.regexp "", "@\"o") ;
-    (Str.regexp "", "@^i") ;
-    (Str.regexp "", "@\"i") ;
-    (Str.regexp "", "@`u") ;
-    (Str.regexp "", "@^u") ;
-    (Str.regexp "", "@ae{}" ) ;
-    (Str.regexp "", "@AE{}" ) ;
-    (Str.regexp "", "@ss{}" ) ;
-    (Str.regexp "", "@copyright{}" ) ;
+    (Str.regexp "\xE0", "@`a") ;
+    (Str.regexp "\xE2", "@^a") ;
+    (Str.regexp "\xE9", "@'e") ;
+    (Str.regexp "\xE8", "@`e") ;
+    (Str.regexp "\xEA", "@^e") ;
+    (Str.regexp "\xEB", "@\"e") ;
+    (Str.regexp "\xF7", "@,{c}") ;
+    (Str.regexp "\xF4", "@^o") ;
+    (Str.regexp "\xF6", "@\"o") ;
+    (Str.regexp "\xEE", "@^i") ;
+    (Str.regexp "\xEF", "@\"i") ;
+    (Str.regexp "\xF9", "@`u") ;
+    (Str.regexp "\xFB", "@^u") ;
+    (Str.regexp "\xE6", "@ae{}" ) ;
+    (Str.regexp "\xC6", "@AE{}" ) ;
+    (Str.regexp "\xDF", "@ss{}" ) ;
+    (Str.regexp "\xA9", "@copyright{}" ) ;
     ]
     else [])
 
@@ -218,7 +224,7 @@ struct
     String.concat "\n"
       [ "@ifinfo" ; s ; "@end ifinfo" ; "" ]
 
-  (** [install-info] informations *)
+  (** [install-info] information *)
   let dirsection sec =
     "@dircategory " ^ (escape sec)
 
@@ -294,15 +300,19 @@ class text =
       | Block t -> self#texi_of_Block t
       | Title (n, _, t) -> self#texi_of_Title n t
       | Link (s, t) -> self#texi_of_Link s t
-      | Ref (name, kind) ->self#texi_of_Ref name kind
+      | Ref (name, kind, _) ->self#texi_of_Ref name kind
       | Superscript t -> self#texi_of_Superscript t
       | Subscript t -> self#texi_of_Subscript t
       | Odoc_info.Module_list _ -> ""
       | Odoc_info.Index_list -> ""
       | Odoc_info.Custom (s,t) -> self#texi_of_custom_text s t
+      | Odoc_info.Target (target, code) -> self#texi_of_Target ~target ~code
 
     method texi_of_custom_text s t = ""
 
+    method texi_of_Target ~target ~code =
+      if String.lowercase target = "texi" then code else ""
+
     method texi_of_Verbatim s = s
     method texi_of_Raw s = self#escape s
     method texi_of_Code s = "@code{" ^ (self#escape s) ^ "}"
@@ -377,6 +387,9 @@ class text =
 
 exception Aliased_node
 
+module Generator =
+struct
+
 (** This class is used to create objects which can generate a simple
     Texinfo documentation. *)
 class texi =
@@ -409,7 +422,7 @@ class texi =
 
     method index (ind : indices) ent =
       Verbatim
-        (if !Args.with_index
+        (if !Global.with_index
         then (assert(List.mem ind indices_to_build) ;
               String.concat ""
                 [ "@" ; indices ind ; "index " ;
@@ -461,6 +474,11 @@ class texi =
                    Raw " " ; Raw s ] @ t @ [ Newline ])
            see_l)
 
+    method text_of_before l =
+      List.flatten
+      (List.map
+        (fun x -> linebreak :: (to_text#text_of_before [x])) l)
+
     method text_of_params params_list =
         List.concat
           (List.map
@@ -470,7 +488,7 @@ class texi =
                  Raw " " ; Raw s ; Raw ": " ] @ t @ [ Newline ] )
              params_list)
 
-    method text_of_raised_exceptions = function
+    method! text_of_raised_exceptions = function
       | [] -> []
       | (s, t) :: [] ->
           [ linebreak ;
@@ -486,12 +504,12 @@ class texi =
                  (fun (ex, desc) ->(Code ex) :: (Raw " ") :: desc ) l ) ;
             Newline ]
 
-    method text_of_return_opt = function
+    method! text_of_return_opt = function
       | None -> []
       | Some t ->
           (Bold [Raw Odoc_messages.returns ]) :: Raw " " :: t @ [ Newline ]
 
-    method text_of_custom c_l =
+    method! text_of_custom c_l =
       List.flatten
         (List.rev
            (List.fold_left
@@ -505,7 +523,7 @@ class texi =
                  acc
               ) [] c_l))
 
-    method text_of_info ?(block=false) = function
+    method! text_of_info ?(block=false) = function
       | None -> []
       | Some info ->
           let t =
@@ -526,6 +544,7 @@ class texi =
                           self#text_of_version_opt info.i_version )
                    else [] ;
                    self#text_of_sees_opt info.i_sees ;
+                   self#text_of_before info.i_before ;
                    if is info.i_since
                    then ( linebreak ::
                           self#text_of_since_opt info.i_since )
@@ -555,7 +574,7 @@ class texi =
              (self#relative_idents m_name
                 (Odoc_info.string_of_type_expr typ)))
 
-    method text_of_short_type_expr m_name typ =
+    method! text_of_short_type_expr m_name typ =
       [ Raw (self#normal_type m_name typ) ]
 
     (** Return Texinfo code for a value. *)
@@ -577,6 +596,7 @@ class texi =
       let t = [ self#fixedblock
                   [ Newline ; minus ;
                     Raw "val " ;
+                    Raw (if a.att_virtual then "virtual " else "") ;
                     Raw (if a.att_mutable then "mutable " else "") ;
                     Raw (Name.simple a.att_value.val_name) ;
                     Raw " :\n" ;
@@ -619,9 +639,13 @@ class texi =
           Printf.sprintf "(%s) "
             (String.concat ", " (List.map f l))
 
-    method string_of_type_args = function
-      | [] -> ""
-      | args -> " of " ^ (Odoc_info.string_of_type_list " * " args)
+    method string_of_type_args (args:Types.type_expr list) (ret:Types.type_expr option) =
+      match args, ret with
+      | [], None -> ""
+      | args, None -> " of " ^ (Odoc_info.string_of_type_list " * " args)
+      | [], Some r -> " : " ^ (Odoc_info.string_of_type_expr r)
+      | args, Some r -> " : " ^ (Odoc_info.string_of_type_list " * " args) ^
+                                " -> " ^ (Odoc_info.string_of_type_expr r)
 
     (** Return Texinfo code for a type. *)
     method texi_of_type ty =
@@ -631,28 +655,32 @@ class texi =
           [ Newline ; minus ; Raw "type " ;
             Raw (self#string_of_type_parameters ty) ;
             Raw (Name.simple ty.ty_name) ] @
+          let priv = ty.ty_private = Asttypes.Private in
           ( match ty.ty_manifest with
           | None -> []
           | Some typ ->
-              (Raw " = ") :: (self#text_of_short_type_expr
-                                (Name.father ty.ty_name) typ) ) @
+              (Raw " = ") ::
+              (Raw (if priv then "private " else "")) ::
+              (self#text_of_short_type_expr (Name.father ty.ty_name) typ) ) @
           (
            match ty.ty_kind with
            | Type_abstract -> [ Newline ]
-           | Type_variant (l, priv) ->
+           | Type_variant l ->
                (Raw (" ="^(if priv then " private" else "")^"\n")) ::
                (List.flatten
                   (List.map
                      (fun constr ->
                        (Raw ("  | " ^ constr.vc_name)) ::
-                       (Raw (self#string_of_type_args constr.vc_args)) ::
+                       (Raw (self#string_of_type_args
+                               constr.vc_args constr.vc_ret)) ::
                        (match constr.vc_text with
                        | None -> [ Newline ]
                        | Some t ->
-                           ((Raw (indent 5 "\n(* ")) :: (self#soft_fix_linebreaks 8 t)) @
+                           (Raw (indent 5 "\n(* ") ::
+                            self#soft_fix_linebreaks 8 t) @
                            [ Raw " *)" ; Newline ]
                        ) ) l ) )
-           | Type_record (l, priv) ->
+           | Type_record l ->
                (Raw (" = "^(if priv then "private " else "")^"{\n")) ::
                (List.flatten
                   (List.map
@@ -681,7 +709,7 @@ class texi =
         [ self#fixedblock
             ( [ Newline ; minus ; Raw "exception " ;
                 Raw (Name.simple e.ex_name) ;
-                Raw (self#string_of_type_args e.ex_args) ] @
+                Raw (self#string_of_type_args e.ex_args None) ] @
               (match e.ex_alias with
               | None -> []
               | Some ea -> [ Raw " = " ; Raw
@@ -713,7 +741,7 @@ class texi =
                 then " = " ^ (resolve_alias_name m)
                 else "" ) ] ] ;
           ( if is_alias_there m
-          then [ Ref (resolve_alias_name m, Some RK_module) ;
+          then [ Ref (resolve_alias_name m, Some RK_module, None) ;
                  Newline ; ]
           else [] ) ;
           ( if is_alias m
@@ -736,13 +764,13 @@ class texi =
         | { mt_name = name } -> name in
       let t =
         [ [ self#fixedblock
-              [ Newline ; minus ; Raw "module type" ;
+              [ Newline ; minus ; Raw "module type " ;
                 Raw (Name.simple mt.mt_name) ;
                 Raw (if is_alias mt
                 then " = " ^ (resolve_alias_name mt)
                 else "" ) ] ] ;
           ( if is_alias_there mt
-          then [ Ref (resolve_alias_name mt, Some RK_module_type) ;
+          then [ Ref (resolve_alias_name mt, Some RK_module_type, None) ;
                  Newline ; ]
           else [] ) ;
           ( if is_alias mt
@@ -761,10 +789,10 @@ class texi =
                         [ Raw im.im_name ]
                     | Some (Mod { m_name = name }) ->
                         [ Raw name ; Raw "\n     " ;
-                          Ref (name, Some RK_module) ]
+                          Ref (name, Some RK_module, None) ]
                     | Some (Modtype { mt_name = name }) ->
                         [ Raw name ; Raw "\n     " ;
-                          Ref (name, Some RK_module_type) ]
+                          Ref (name, Some RK_module_type, None) ]
                     ) @
                    [ Newline ] @
                    (self#text_of_info im.im_info)
@@ -779,7 +807,7 @@ class texi =
       let t = [ self#fixedblock
                   [ Newline ; minus ; Raw "class " ;
                     Raw (Name.simple c.cl_name) ] ;
-                Ref (c.cl_name, Some RK_class) ; Newline ;
+                Ref (c.cl_name, Some RK_class, None) ; Newline ;
                 Newline ] @ (self#text_of_info c.cl_info) in
       self#texi_of_text t
 
@@ -789,7 +817,7 @@ class texi =
       let t = [ self#fixedblock
                   [ Newline ; minus ; Raw "class type " ;
                     Raw (Name.simple ct.clt_name) ] ;
-                Ref (ct.clt_name, Some RK_class_type) ; Newline ;
+                Ref (ct.clt_name, Some RK_class_type, None) ; Newline ;
                 Newline ] @ (self#text_of_info ct.clt_info) in
       self#texi_of_text t
 
@@ -833,7 +861,7 @@ class texi =
               | Cl _ -> Some RK_class
               | Cltype _ -> Some RK_class_type in
             (Code inh.ic_name) ::
-            (Ref (inh.ic_name, kind)) ::
+            (Ref (inh.ic_name, kind, None)) ::
             ( match inh.ic_text with
             | None -> []
             | Some t -> Newline :: t)
@@ -993,12 +1021,12 @@ class texi =
       let title = [
         self#node depth m.m_name ;
         Title (depth, None,
-	       if m.m_text_only then
-		 [ Raw m.m_name ]
-	       else
-		 [ Raw (Odoc_messages.modul ^ " ") ;
+               if m.m_text_only then
+                 [ Raw m.m_name ]
+               else
+                 [ Raw (Odoc_messages.modul ^ " ") ;
                    Code m.m_name ]
-	      ) ;
+              ) ;
         self#index `Module m.m_name ; Newline ] in
       puts chanout (self#texi_of_text title) ;
 
@@ -1042,7 +1070,7 @@ class texi =
 
     (** Writes the header of the TeXinfo document. *)
     method generate_texi_header chan texi_filename m_list =
-      let title = match !Args.title with
+      let title = match !Global.title with
       | None -> ""
       | Some s -> self#escape s in
       let filename =
@@ -1067,18 +1095,18 @@ class texi =
                "@settitle " ^ title ;
                "@c %**end of header" ; ] ;
 
-             (if !Args.with_index then
+             (if !Global.with_index then
                List.map
                  (fun ind ->
                    "@defcodeindex " ^ (indices ind))
                  indices_to_build
              else []) ;
 
-             [ Texi.dirsection !Args.info_section ] ;
+             [ Texi.dirsection !info_section ] ;
 
              Texi.direntry
-               (if !Args.info_entry <> []
-               then !Args.info_entry
+               (if !info_entry <> []
+               then !info_entry
                else [ Printf.sprintf "* %s: (%s)."
                         title
                         (Filename.chop_suffix filename ".info") ]) ;
@@ -1095,7 +1123,7 @@ class texi =
 
       (* insert the intro file *)
       begin
-        match !Odoc_info.Args.intro_file with
+        match !Odoc_info.Global.intro_file with
         | None when title <> "" ->
             puts_nl chan "@ifinfo" ;
             puts_nl chan ("Documentation for " ^ title) ;
@@ -1106,13 +1134,13 @@ class texi =
             nl chan ;
             puts_nl chan
               (self#texi_of_info
-		 (Some (Odoc_info.info_of_comment_file m_list f)))
+                 (Some (Odoc_info.info_of_comment_file m_list f)))
       end ;
 
       (* write a top menu *)
       Texi.generate_menu chan
         ((List.map (fun m -> `Module m) m_list) @
-         (if !Args.with_index then
+         (if !Global.with_index then
            let indices_names_to_build = List.map indices indices_to_build in
            List.rev
              (List.fold_left
@@ -1129,7 +1157,7 @@ class texi =
     (** Writes the trailer of the TeXinfo document. *)
     method generate_texi_trailer chan =
       nl chan ;
-      if !Args.with_index
+      if !Global.with_index
       then
         let indices_names_to_build = List.map indices indices_to_build in
         List.iter (puts_nl chan)
@@ -1142,7 +1170,7 @@ class texi =
                          "@printindex " ^ shortname ; ]
                   else [])
                 indices_names )) ;
-      if !Args.with_toc
+      if !Global.with_toc
       then puts_nl chan "@contents" ;
       puts_nl chan "@bye"
 
@@ -1190,25 +1218,25 @@ class texi =
 
 
     (** Generate the Texinfo file from a module list,
-       in the {!Odoc_info.Args.out_file} file. *)
+       in the {!Odoc_info.Global.out_file} file. *)
     method generate module_list =
       Hashtbl.clear node_tbl ;
       let filename =
-        if !Args.out_file = Odoc_messages.default_out_file
+        if !Global.out_file = Odoc_messages.default_out_file
         then "ocamldoc.texi"
-        else !Args.out_file in
-      if !Args.with_index
+        else !Global.out_file in
+      if !Global.with_index
       then List.iter self#scan_for_index
           (List.map (fun m -> `Module m) module_list) ;
       try
         let chanout = open_out
-            (Filename.concat !Args.target_dir filename) in
-        if !Args.with_header
+            (Filename.concat !Global.target_dir filename) in
+        if !Global.with_header
         then self#generate_texi_header chanout filename module_list ;
         List.iter
           (self#generate_for_module chanout)
           module_list ;
-        if !Args.with_trailer
+        if !Global.with_trailer
         then self#generate_texi_trailer chanout ;
         close_out chanout
       with
@@ -1217,3 +1245,6 @@ class texi =
           prerr_endline s ;
           incr Odoc_info.errors
   end
+end
+
+module type Texi_generator = module type of Generator
