@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -10,8 +10,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: subst.ml,v 1.52 2008/01/11 16:13:16 doligez Exp $ *)
-
 (* Substitutions *)
 
 open Misc
@@ -19,7 +17,7 @@ open Path
 open Types
 open Btype
 
-type t = 
+type t =
   { types: (Ident.t, Path.t) Tbl.t;
     modules: (Ident.t, Path.t) Tbl.t;
     modtypes: (Ident.t, module_type) Tbl.t;
@@ -45,6 +43,18 @@ let rec module_path s = function
   | Papply(p1, p2) ->
       Papply(module_path s p1, module_path s p2)
 
+let modtype_path s = function
+    Pident id as p ->
+      begin try
+        match Tbl.find id s.modtypes with
+          | Mty_ident p -> p
+          | _ -> fatal_error "Subst.modtype_path"
+      with Not_found -> p end
+  | Pdot(p, n, pos) ->
+      Pdot(module_path s p, n, pos)
+  | Papply(p1, p2) ->
+      fatal_error "Subst.modtype_path"
+
 let type_path s = function
     Pident id as p ->
       begin try Tbl.find id s.types with Not_found -> p end
@@ -59,18 +69,20 @@ let new_id = ref (-1)
 let reset_for_saving () = new_id := -1
 
 let newpersty desc =
-  decr new_id; { desc = desc; level = generic_level; id = !new_id }
+  decr new_id;
+  { desc = desc; level = generic_level; id = !new_id }
 
 (* Similar to [Ctype.nondep_type_rec]. *)
 let rec typexp s ty =
   let ty = repr ty in
   match ty.desc with
-    Tvar | Tunivar ->
+    Tvar _ | Tunivar _ as desc ->
       if s.for_saving || ty.id < 0 then
         let ty' =
-          if s.for_saving then newpersty ty.desc else newty2 ty.level ty.desc
+          if s.for_saving then newpersty desc
+          else newty2 ty.level desc
         in
-        save_desc ty ty.desc; ty.desc <- Tsubst ty'; ty'
+        save_desc ty desc; ty.desc <- Tsubst ty'; ty'
       else ty
   | Tsubst ty ->
       ty
@@ -82,18 +94,24 @@ let rec typexp s ty =
     let desc = ty.desc in
     save_desc ty desc;
     (* Make a stub *)
-    let ty' = if s.for_saving then newpersty Tvar else newgenvar () in
+    let ty' = if s.for_saving then newpersty (Tvar None) else newgenvar () in
     ty.desc <- Tsubst ty';
     ty'.desc <-
       begin match desc with
       | Tconstr(p, tl, abbrev) ->
           Tconstr(type_path s p, List.map (typexp s) tl, ref Mnil)
+      | Tpackage(p, n, tl) ->
+          Tpackage(modtype_path s p, n, List.map (typexp s) tl)
       | Tobject (t1, name) ->
           Tobject (typexp s t1,
                  ref (match !name with
                         None -> None
                       | Some (p, tl) ->
                           Some (type_path s p, List.map (typexp s) tl)))
+      | Tfield (m, k, t1, t2)
+        when s == identity && ty.level < generic_level && m = dummy_method ->
+          (* not allowed to lower the level of the dummy method *)
+          Tfield (m, k, t1, typexp s t2)
       | Tvariant row ->
           let row = row_repr row in
           let more = repr row.row_more in
@@ -112,11 +130,11 @@ let rec typexp s ty =
               let more' =
                 match more.desc with
                   Tsubst ty -> ty
-                | Tconstr _ -> typexp s more
-                | Tunivar | Tvar ->
+                | Tconstr _ | Tnil -> typexp s more
+                | Tunivar _ | Tvar _ ->
                     save_desc more more.desc;
                     if s.for_saving then newpersty more.desc else
-                    if dup && more.desc <> Tunivar then newgenvar () else more
+                    if dup && is_Tvar more then newgenty more.desc else more
                 | _ -> assert false
               in
               (* Register new type first for recursion *)
@@ -153,22 +171,26 @@ let type_declaration s decl =
         begin match decl.type_kind with
           Type_abstract -> Type_abstract
         | Type_variant cstrs ->
-            Type_variant(
-              List.map (fun (n, args) -> (n, List.map (typexp s) args))
-                       cstrs)
+            Type_variant
+              (List.map
+                 (fun (n, args, ret_type) ->
+                   (n, List.map (typexp s) args, may_map (typexp s) ret_type))
+                 cstrs)
         | Type_record(lbls, rep) ->
-            Type_record(
-              List.map (fun (n, mut, arg) -> (n, mut, typexp s arg))
-                       lbls,
-              rep)
+            Type_record
+              (List.map (fun (n, mut, arg) -> (n, mut, typexp s arg)) lbls,
+               rep)
         end;
       type_manifest =
-        begin match decl.type_manifest with
-          None -> None
-        | Some ty -> Some(typexp s ty)
+        begin
+          match decl.type_manifest with
+            None -> None
+          | Some ty -> Some(typexp s ty)
         end;
       type_private = decl.type_private;
       type_variance = decl.type_variance;
+      type_newtype_level = None;
+      type_loc = if s.for_saving then Location.none else decl.type_loc;
     }
   in
   cleanup_types ();
@@ -186,12 +208,12 @@ let class_signature s sign =
 
 let rec class_type s =
   function
-    Tcty_constr (p, tyl, cty) ->
-      Tcty_constr (type_path s p, List.map (typexp s) tyl, class_type s cty)
-  | Tcty_signature sign ->
-      Tcty_signature (class_signature s sign)
-  | Tcty_fun (l, ty, cty) ->
-      Tcty_fun (l, typexp s ty, class_type s cty)
+    Cty_constr (p, tyl, cty) ->
+      Cty_constr (type_path s p, List.map (typexp s) tyl, class_type s cty)
+  | Cty_signature sign ->
+      Cty_signature (class_signature s sign)
+  | Cty_fun (l, ty, cty) ->
+      Cty_fun (l, typexp s ty, class_type s cty)
 
 let class_declaration s decl =
   let decl =
@@ -227,43 +249,47 @@ let class_type s cty =
 
 let value_description s descr =
   { val_type = type_expr s descr.val_type;
-    val_kind = descr.val_kind }
+    val_kind = descr.val_kind;
+    val_loc = if s.for_saving then Location.none else descr.val_loc;
+   }
 
-let exception_declaration s tyl =
-  List.map (type_expr s) tyl
+let exception_declaration s descr =
+  { exn_args = List.map (type_expr s) descr.exn_args;
+    exn_loc = if s.for_saving then Location.none else descr.exn_loc;
+   }
 
 let rec rename_bound_idents s idents = function
     [] -> (List.rev idents, s)
-  | Tsig_type(id, d, _) :: sg ->
+  | Sig_type(id, d, _) :: sg ->
       let id' = Ident.rename id in
       rename_bound_idents (add_type id (Pident id') s) (id' :: idents) sg
-  | Tsig_module(id, mty, _) :: sg ->
+  | Sig_module(id, mty, _) :: sg ->
       let id' = Ident.rename id in
       rename_bound_idents (add_module id (Pident id') s) (id' :: idents) sg
-  | Tsig_modtype(id, d) :: sg ->
+  | Sig_modtype(id, d) :: sg ->
       let id' = Ident.rename id in
-      rename_bound_idents (add_modtype id (Tmty_ident(Pident id')) s)
+      rename_bound_idents (add_modtype id (Mty_ident(Pident id')) s)
                           (id' :: idents) sg
-  | (Tsig_value(id, _) | Tsig_exception(id, _) | 
-     Tsig_class(id, _, _) | Tsig_cltype(id, _, _)) :: sg ->
+  | (Sig_value(id, _) | Sig_exception(id, _) |
+     Sig_class(id, _, _) | Sig_class_type(id, _, _)) :: sg ->
       let id' = Ident.rename id in
       rename_bound_idents s (id' :: idents) sg
 
 let rec modtype s = function
-    Tmty_ident p as mty ->
+    Mty_ident p as mty ->
       begin match p with
         Pident id ->
           begin try Tbl.find id s.modtypes with Not_found -> mty end
       | Pdot(p, n, pos) ->
-          Tmty_ident(Pdot(module_path s p, n, pos))
+          Mty_ident(Pdot(module_path s p, n, pos))
       | Papply(p1, p2) ->
           fatal_error "Subst.modtype"
       end
-  | Tmty_signature sg ->
-      Tmty_signature(signature s sg)
-  | Tmty_functor(id, arg, res) ->
+  | Mty_signature sg ->
+      Mty_signature(signature s sg)
+  | Mty_functor(id, arg, res) ->
       let id' = Ident.rename id in
-      Tmty_functor(id', modtype s arg,
+      Mty_functor(id', modtype s arg,
                         modtype (add_module id (Pident id') s) res)
 
 and signature s sg =
@@ -276,21 +302,36 @@ and signature s sg =
 
 and signature_component s comp newid =
   match comp with
-    Tsig_value(id, d) ->
-      Tsig_value(newid, value_description s d)
-  | Tsig_type(id, d, rs) ->
-      Tsig_type(newid, type_declaration s d, rs)
-  | Tsig_exception(id, d) ->
-      Tsig_exception(newid, exception_declaration s d)
-  | Tsig_module(id, mty, rs) ->
-      Tsig_module(newid, modtype s mty, rs)
-  | Tsig_modtype(id, d) ->
-      Tsig_modtype(newid, modtype_declaration s d)
-  | Tsig_class(id, d, rs) ->
-      Tsig_class(newid, class_declaration s d, rs)
-  | Tsig_cltype(id, d, rs) ->
-      Tsig_cltype(newid, cltype_declaration s d, rs)
+    Sig_value(id, d) ->
+      Sig_value(newid, value_description s d)
+  | Sig_type(id, d, rs) ->
+      Sig_type(newid, type_declaration s d, rs)
+  | Sig_exception(id, d) ->
+      Sig_exception(newid, exception_declaration s d)
+  | Sig_module(id, mty, rs) ->
+      Sig_module(newid, modtype s mty, rs)
+  | Sig_modtype(id, d) ->
+      Sig_modtype(newid, modtype_declaration s d)
+  | Sig_class(id, d, rs) ->
+      Sig_class(newid, class_declaration s d, rs)
+  | Sig_class_type(id, d, rs) ->
+      Sig_class_type(newid, cltype_declaration s d, rs)
 
 and modtype_declaration s = function
-    Tmodtype_abstract -> Tmodtype_abstract
-  | Tmodtype_manifest mty -> Tmodtype_manifest(modtype s mty)
+    Modtype_abstract -> Modtype_abstract
+  | Modtype_manifest mty -> Modtype_manifest(modtype s mty)
+
+(* For every binding k |-> d of m1, add k |-> f d to m2
+   and return resulting merged map. *)
+
+let merge_tbls f m1 m2 =
+  Tbl.fold (fun k d accu -> Tbl.add k (f d) accu) m1 m2
+
+(* Composition of substitutions:
+     apply (compose s1 s2) x = apply s2 (apply s1 x) *)
+
+let compose s1 s2 =
+  { types = merge_tbls (type_path s2) s1.types s2.types;
+    modules = merge_tbls (module_path s2) s1.modules s2.modules;
+    modtypes = merge_tbls (modtype s2) s1.modtypes s2.modtypes;
+    for_saving = false }
