@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -10,8 +10,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: subst.mli 9547 2010-01-22 12:48:24Z doligez $ *)
-
 (* Substitutions *)
 
 open Types
@@ -48,7 +46,7 @@ val type_declaration: t -> type_declaration -> type_declaration
 val exception_declaration:
         t -> exception_declaration -> exception_declaration
 val class_declaration: t -> class_declaration -> class_declaration
-val cltype_declaration: t -> cltype_declaration -> cltype_declaration
+val cltype_declaration: t -> class_type_declaration -> class_type_declaration
 val modtype: t -> module_type -> module_type
 val signature: t -> signature -> signature
 val modtype_declaration: t -> modtype_declaration -> modtype_declaration
